@@ -164,17 +164,19 @@ DatabaseManager::DatabaseManager(QObject *parent) :
 
 DatabaseManager::~DatabaseManager()
 {
+	qDebug () << "CLOSE DB " << connectionName;
 	if (connectionName.length() > 0)
 		db.removeDatabase(connectionName);
 //	else
 //		db.removeDatabase();
 
-	//qDebug () << "CLOSE DB " << connectionName;
+	qDebug () << "CLOSED DB " << connectionName;
+
 	if (db.isOpen()) {
 		qDebug () << "CLOSing DB " << connectionName;
 		db.close();
 	}
-	opendb = false;
+
 }
 
 bool DatabaseManager::close()
@@ -340,6 +342,222 @@ bool DatabaseManager::restoreDB(const QString & prefix) {
 //	return _openDB ("qtwallet.db.sqlite");
 //}
 
+bool DatabaseManager::_mergeDB (DatabaseManager * mergeDB) {
+    QList<Target*>  listMerge = mergeDB->getTargets(NULL);
+    QList<Target*>  list = getTargets(NULL);
+
+    int maxOrig = 0;
+    for (int i = 0; i < list.size (); i++) {
+        Target * target = list[i];
+        if (target->id > maxOrig)
+            maxOrig = target->id;
+    }
+
+    qDebug () << "max orig is " << maxOrig;
+
+    int maxMerg = 0;
+    for (int j = 0; j < listMerge.size (); j++) {
+        Target * targetMerge = listMerge[j];
+        if (targetMerge->id > maxMerg)
+            maxMerg = targetMerge->id;
+    }
+
+    qDebug () << "max merg is " << maxMerg;
+
+    // TODO check remaining elements on listMerge
+    bool res;
+
+    for (int i = 0; i < list.size (); i++) {
+        Target * target = list[i];
+
+        qDebug () << "Checking[" << i << "] " << target->id;
+        int j = 0;
+        for (; j < listMerge.size (); j++) {
+            Target * targetMerge = listMerge[j];
+            qDebug () << "Looking for[" << j << "] " << targetMerge->id;
+            if (targetMerge->id == target->id) {
+
+                qDebug () << "Checking changes with[" << j << "]";
+
+                if (targetMerge->creationdate != target->creationdate) {
+                    qDebug () << "Attention different creation time[" << j << "]";
+
+                    int idTargetMerge = targetMerge->id;
+                    int idTarget= target->id;
+
+                    if (maxOrig >= maxMerg) {
+                        mergeDB->getFields(targetMerge);
+                        targetMerge->getPicture(mergeDB);
+                        targetMerge->getPictureBack(mergeDB);
+                        res = this->insertTarget(targetMerge);
+                        this->updateFields(targetMerge);
+
+                        qDebug () << "New target on this [" << targetMerge->id << "]";
+
+                        res = mergeDB->copyTarget(targetMerge);
+                        mergeDB->updateFields(targetMerge);
+
+                        qDebug () << "New target on merged [" << targetMerge->id << "]";
+
+                        this->getFields(target);
+                        target->getPicture(this);
+                        target->getPictureBack(this);
+                        res = mergeDB->insertTarget(target);
+                        mergeDB->updateFields(target);
+
+                        qDebug () << "New target on merged [" << target->id << "]";
+
+                        res = this->copyTarget(target);
+                        this->updateFields(target);
+
+                        qDebug () << "New target on this [" << target->id << "]";
+                    }
+                    else {
+                        this->getFields(target);
+                        target->getPicture(this);
+                        target->getPictureBack(this);
+                        res = mergeDB->insertTarget(target);
+                        mergeDB->updateFields(target);
+
+                        qDebug () << "New target on merged [" << target->id << "]";
+
+                        res = this->copyTarget(target);
+                        this->updateFields(target);
+
+                        qDebug () << "New target on this [" << target->id << "]";
+
+                        mergeDB->getFields(targetMerge);
+                        targetMerge->getPicture(mergeDB);
+                        targetMerge->getPictureBack(mergeDB);
+                        res = this->insertTarget(targetMerge);
+                        this->updateFields(targetMerge);
+
+                        qDebug () << "New target on this [" << targetMerge->id << "]";
+
+                        res = mergeDB->copyTarget(targetMerge);
+                        mergeDB->updateFields(targetMerge);
+
+                        qDebug () << "New target on merged [" << targetMerge->id << "]";
+                    }
+                    mergeDB->_deleteTarget (idTargetMerge);
+                    this->_deleteTarget (idTarget);
+
+                    target->freeFieldList();
+                    target->freeResources();
+
+                    targetMerge->freeFieldList();
+                    targetMerge->freeResources();
+
+                    break;
+                }
+
+                if (targetMerge->modificationdate < target->modificationdate) {
+                    qDebug () << "MOVE << [" << j << "]";
+
+                    targetMerge->name = target->name;
+                    targetMerge->groupName = target->groupName;
+                    targetMerge->setPicture(target->getPicture(this));
+                    targetMerge->setPictureBack(target->getPictureBack(this));
+                    //??targetMerge->creationdate = target->creationdate;
+                    targetMerge->modificationdate = target->modificationdate;
+
+                    // TODO merge fields?????
+                    // Load fields from this into targetMerge
+                    this->getFields(targetMerge);
+
+                    mergeDB->updateTarget(targetMerge);
+                    mergeDB->updateTargetPicture(targetMerge);
+                    mergeDB->updateTargetPictureBack(targetMerge);
+                    mergeDB->updateFields(targetMerge);
+
+                }
+                else
+                if (targetMerge->modificationdate > target->modificationdate) {
+                    qDebug () << "MOVE >> [" << j << "]";
+
+                    // TODO is required to duplicate images and strings
+                    target->name = targetMerge->name;
+                    target->groupName = targetMerge->groupName;
+                    target->setPicture(targetMerge->getPicture(mergeDB));
+                    target->setPictureBack(targetMerge->getPictureBack(mergeDB));
+                    //??target->creationdate = targetMerge->creationdate;
+                    target->modificationdate = targetMerge->modificationdate;
+
+                    // TODO merge fields?????
+                    // Load fields from mergeDB into target
+                    mergeDB->getFields(target);
+
+                    this->updateTarget(target);
+                    this->updateTargetPicture(target);
+                    this->updateTargetPictureBack(target);
+                    this->updateFields(target);
+                }
+                else {
+                    qDebug () << "SAME TIME[" << j << "]";
+                }
+
+                target->freeFieldList();
+                target->freeResources();
+
+                targetMerge->freeFieldList();
+                targetMerge->freeResources();
+                break;
+            }
+        }
+        if (j > listMerge.size ()) {
+            // NOT FOUND
+
+            this->getFields(target);
+            target->getPicture(this);
+            target->getPictureBack(this);
+            res = mergeDB->copyTarget(target);
+            mergeDB->updateFields(target);
+
+            qDebug () << "New target on merged [" << target->id << "]";
+
+            target->freeFieldList();
+            target->freeResources();
+        }
+        else {
+            Target * targetMerge = listMerge[j];
+            listMerge.removeAt(j);
+            delete targetMerge;
+
+            qDebug () << "remaining " << listMerge.size () << " elements on merge list";
+        }
+    }
+
+    int j = 0;
+    for (; j < listMerge.size (); j++) {
+        Target * targetMerge = listMerge[j];
+
+        mergeDB->getFields(targetMerge);
+        targetMerge->getPicture(mergeDB);
+        targetMerge->getPictureBack(mergeDB);
+        res = this->copyTarget(targetMerge);
+        this->updateFields(targetMerge);
+
+        qDebug () << "New target on this [" << targetMerge->id << "]";
+
+        targetMerge->freeFieldList();
+        targetMerge->freeResources();
+    }
+
+    Target * target;
+    foreach(target, list) {
+            delete target;
+    }
+    foreach(target, listMerge) {
+            delete target;
+    }
+    // TODO should I delete objects on list???
+    listMerge.clear();
+    list.clear();
+
+    return true;
+}
+
+
 bool DatabaseManager::migrationFrom (DatabaseManager * mgr) {
 	bool res;
 
@@ -381,8 +599,27 @@ bool DatabaseManager::migrationFrom (DatabaseManager * mgr) {
 	return res;
 }
 
+bool DatabaseManager::_openFileDB(const QString & name, const QString & cName) {
+        qDebug () << "OPEN DB " << name << "(" << cName << ")";
+
+        connectionName = cName;
+
+        // Find QSLite driver
+        if (connectionName.length() > 0)
+                db = QSqlDatabase::addDatabase("QSQLITE", connectionName);
+        else
+                db = QSqlDatabase::addDatabase("QSQLITE");
+
+        db.setDatabaseName(name);
+
+        if (db.open()) {
+                return true;
+        }
+        return false;
+}
+
 bool DatabaseManager::_openDB(const QString & name, const QString & cName) {
-	//qDebug () << "OPEN DB " << name << "(" << cName << ")";
+        qDebug () << "OPEN DB " << name << "(" << cName << ")";
 
 	connectionName = cName;
 
@@ -399,9 +636,6 @@ bool DatabaseManager::_openDB(const QString & name, const QString & cName) {
 
 	db.setDatabaseName(path);
 
-	// Open databasee
-	bool res = false;
-
 	if (db.open()) {
 		return true;
 	}
@@ -494,6 +728,46 @@ void DatabaseManager::reset () {
 
 }
 
+
+QString DatabaseManager::mergeDB (const QString & dbToMerge, const QString & passwordToMerge) {
+    bool ret = false;
+
+    QString result = "";
+
+    ret = backupDB("BACK"); // TODO remove
+
+    if (ret) {
+        DatabaseManager * mergeDB = new DatabaseManager();
+        ret = mergeDB->backupDB("BACKMERGE"); // TODO remove
+
+        ret = mergeDB->_openFileDB(dbToMerge, "MERGE");
+        if (ret) {
+            int currentVersionToMerge =  mergeDB->getVersion ();
+            if (currentVersionToMerge != SOFT_VERSION) {
+                qDebug () << "INVALID VERSION " << currentVersionToMerge;
+            }
+            else {
+                if (hasSecretPhrase()) {
+                    if (mergeDB->checkPassword(passwordToMerge, SOFT_VERSION)) {
+                        ret = _mergeDB (mergeDB);
+                    }
+                    else {
+                        qDebug () << "INVALID PASSWORD";
+                    }
+                }
+                else {
+                    qDebug () << "NOT INIT DATABASE";
+                }
+            }
+
+        }
+
+        delete mergeDB;
+    }
+
+    return "";
+}
+
 QString DatabaseManager::changePassword (const QString & newPassword) {
 	bool ret = false;
 
@@ -1078,12 +1352,17 @@ QList<QString> DatabaseManager::getGroups()
 
 bool DatabaseManager::deleteTarget(Target * target)
 {
-	bool ret = false;
+    return _deleteTarget (target->id);
+}
+
+bool DatabaseManager::_deleteTarget(int idTarget)
+{
+    bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
 
 		ret = query.prepare("delete from fields where targetId=:code");
-		query.bindValue(":code", target->id);
+                query.bindValue(":code", idTarget);
 		ret = query.exec();
 
 		if (ret) {
@@ -1091,7 +1370,7 @@ bool DatabaseManager::deleteTarget(Target * target)
 			if (ret) {
 				//qDebug () << "DELETE " << target->id;
 
-				query.bindValue(":code", target->id);
+                                query.bindValue(":code", idTarget);
 				ret = query.exec();
 			}
 		}
