@@ -3,6 +3,7 @@
  */
 
 #include "utils.h"
+
 #include "eWallet.h"
 #include "LoginDialog.h"
 #include "Constants.h"
@@ -64,6 +65,7 @@ LoginDialog::LoginDialog(bool firstTime, eWallet *paTarget): QWidget((QWidget*)p
 {
 	setContextMenuPolicy(Qt::NoContextMenu);
 
+
 	isFirstTime = firstTime;
 	NTimes = 0;
 
@@ -179,11 +181,15 @@ LoginDialog::LoginDialog(bool firstTime, eWallet *paTarget): QWidget((QWidget*)p
 
 	setLayout(layout);
 
+#ifdef Q_WS_MAEMO_5
 	orientationChanged (0);
+#endif
 }
 
+
 void LoginDialog::orientationChanged(int o) {
-	QRect screenGeometry = QApplication::desktop()->screenGeometry();
+#ifdef Q_WS_MAEMO_5
+    QRect screenGeometry = QApplication::desktop()->screenGeometry();
 	if (screenGeometry.width() > screenGeometry.height()) {
 
 		componentLayout->setColumnMinimumWidth(3, B_SIZE);
@@ -213,7 +219,9 @@ void LoginDialog::orientationChanged(int o) {
 //		}
 		dlgBtnBox->hide ();
 	}
+#endif
 }
+
 LoginDialog::~LoginDialog()
 {
 }
@@ -239,14 +247,17 @@ void LoginDialog::acceptS() {
 	QString current = label->text();
 
 	if (current.length() == 0) {
+#ifdef Q_WS_MAEMO_5
 		show_banner (this, tr("Password is required"));
-//		if (messageBox.isNull()) {
-//			messageBox = new QMessageBox(this);
-//			messageBox->information(this, "Wallet", tr("Password is required"), QMessageBox::Ok);
-//		}
-//		delete messageBox;
-//		messageBox = 0;
-
+#endif
+#ifndef Q_WS_MAEMO_5
+		if (messageBox.isNull()) {
+			messageBox = new QMessageBox(this);
+			messageBox->information(this, "Wallet", tr("Password is required"), QMessageBox::Ok);
+		}
+		delete messageBox;
+		messageBox = 0;
+#endif
 		return;
 	}
 
@@ -257,29 +268,34 @@ void LoginDialog::acceptS() {
 
 			clearS ();
 
+#ifdef Q_WS_MAEMO_5
 			show_banner (this, tr("For verification, please, enter your password again"));
-//			if (messageBox.isNull()) {
-//				messageBox = new QMessageBox(this);
-//				messageBox->information(this, "Wallet", tr("For verification, please, enter your password again"), QMessageBox::Ok);
-//			}
-//			delete messageBox;
-//			messageBox = 0;
-
+#endif
+#ifndef Q_WS_MAEMO_5
+			if (messageBox.isNull()) {
+				messageBox = new QMessageBox(this);
+				messageBox->information(this, "Wallet", tr("For verification, please, enter your password again"), QMessageBox::Ok);
+			}
+			delete messageBox;
+			messageBox = 0;
+#endif
 			return;
 		}
 		if (NTimes == 1) {
 			if (newPassword != current) {
 				NTimes = 0;
 				clearS ();
+#ifdef Q_WS_MAEMO_5
 				show_banner (this, tr("Invalid password, please enter again"));
-
-//				if (messageBox.isNull()) {
-//					messageBox = new QMessageBox(this);
-//					messageBox->information(this, "Wallet", tr("Invalid password, please enter again"), QMessageBox::Ok);
-//				}
-//				delete messageBox;
-//				messageBox = 0;
-
+#endif
+#ifndef Q_WS_MAEMO_5
+				if (messageBox.isNull()) {
+					messageBox = new QMessageBox(this);
+					messageBox->information(this, "Wallet", tr("Invalid password, please enter again"), QMessageBox::Ok);
+				}
+				delete messageBox;
+				messageBox = 0;
+#endif
 				return;
 			}
 		}
