@@ -12,15 +12,17 @@
  * TODO fix problems with key
  * TODO after import groups are not refresh
  * TODO get language from system not from regional settings
- * TODO export to xml
  * TODO import from xml
  * TODO move fields value into a big field
  * TODO what happens with small pictures
   */
 
 #include "utils.h"
+
+#ifdef Q_WS_MAEMO_5
 #include <QDBusInterface>
 #include <QDBusConnection>
+#endif
 
 #include "eWallet.h"
 #include "Constants.h"
@@ -178,9 +180,9 @@ eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 
 
 void eWallet::orientationChanged(int o) {
-	if (isAskPassword) {
-		loginDialogDlg->orientationChanged (o);
-	}
+    if (isAskPassword) {
+        loginDialogDlg->orientationChanged (o);
+    }
 }
 
 void eWallet::showList (bool interactive) {
@@ -383,7 +385,11 @@ void eWallet::listenInactivity(QString s) {
 
 		// TODO sure????
 		fnListWidget->clear();
-		list.clear ();
+                Target * target;
+                foreach(target, list) {
+                        delete target;
+                }
+                list.clear ();
 		grouplist.clear ();
 
 		if (!detailWindow.isNull()) {
@@ -514,6 +520,8 @@ void eWallet::deleteDB () {
 
 bool eWallet::openDataBase (const QString & pas)
 {
+	tempPass = pas;
+
 	bool res = ::dbManager->openDB(pas);
 
 	if (!res) {
@@ -936,8 +944,16 @@ void eWallet::showTargets(bool reset, const QString * filter)
 	fnListWidget->clear();
 
 	if (reset) {
-		list.clear ();
-		list = ::dbManager->getTargets (filter);
+            Target * target;
+            foreach(target, list) {
+                    delete target;
+            }
+//            for (int i = 0; i < list.size (); i++) {
+//                    Target * target = list.at(i);
+//                    delete target;
+//                }
+            list.clear ();
+            list = ::dbManager->getTargets (filter);
 	}
 
 	if (ascSort)
@@ -990,7 +1006,8 @@ void eWallet::createMenu()
 		menuBar()->addAction(tr("Invert Sort"), this, SLOT(sort()));
 		menuBar()->addAction(tr("Import"), this, SLOT(import()));
 		menuBar()->addAction(tr("Export"), this, SLOT(exportData()));
-		//menuBar()->addAction(tr("Export Xml"), this, SLOT(exportXml()));
+		menuBar()->addAction(tr("Export Xml"), this, SLOT(exportXml()));
+		menuBar()->addAction(tr("Merge DB"), this, SLOT(mergeDB()));
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
 		menuBar()->addAction(tr("Change Password"), this, SLOT(changePassword()));
 		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
@@ -1016,8 +1033,11 @@ void eWallet::createMenu()
 		m->addSeparator();
 		m->addAction(tr("Import"), this, SLOT(import()));
 		m->addAction(tr("Export"), this, SLOT(exportData()));
-		//m->addAction(tr("Export Xml"), this, SLOT(exportXml()));
-		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
+		m->addAction(tr("Export Xml"), this, SLOT(exportXml()));
+                m->addSeparator();
+                m->addAction(tr("Merge DB"), this, SLOT(mergeDB()));
+                m->addSeparator();
+                menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
 		menuBar()->addAction(tr("About"), this, SLOT(about()));
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
 	}
@@ -1607,6 +1627,11 @@ void eWallet::exportData() {
 				delete target;
 			}
 
+//                        Target * target;
+//                        foreach(target, list) {
+//                                delete target;
+//                        }
+
 			list.clear();
 
 		    // Close the file
@@ -1618,13 +1643,36 @@ void eWallet::exportData() {
 #endif
 }
 
-QString FIX_NODE (QString a) {
-	// TODO escape characters
-	return a;
-}
 QByteArray FIX_VALUE (QString a) {
 	// TODO escape characters
-	return a.toUtf8();
+	return Qt::escape(a).toUtf8();
+}
+
+void eWallet::mergeDB() {
+    QString path(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
+    QString s = QFileDialog::getOpenFileName(
+                    this,
+                    tr("Choose DB file"),
+                    path,
+                    tr("DB files (*.sqlite);;"));
+
+#ifdef Q_WS_MAEMO_5
+    this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+#endif
+
+    if (!s.isNull () && !s.isEmpty ()) {
+        QFile file(s);
+
+        QString res = ::dbManager->mergeDB(s, tempPass);
+
+        qDebug () <<  "finish merge " << res;
+
+        showInfo (true);
+    }
+
+#ifdef Q_WS_MAEMO_5
+    this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+#endif
 }
 
 void eWallet::exportXml() {
@@ -1659,6 +1707,7 @@ void eWallet::exportXml() {
 				Target * target = list[i];
 
 				t << "<target>\n";
+				t << "<id>" << target->id << "</id>\n";
 				t << "<name>" << FIX_VALUE(target->name) << "</name>\n";
 				t << "<group>" << FIX_VALUE(target->groupName) << "</group>\n";
 				t << "<type>" << FIX_VALUE(target->type) << "</type>\n";
@@ -1691,29 +1740,31 @@ void eWallet::exportXml() {
 				foreach(field, target->fields) {
 	    			  switch (field->type) {
 						  case TYPE_TEXT:
-							t << "<" << FIX_NODE(field->name) << " type=\"T\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"T\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case TYPE_DATE:
-								t << "<" << FIX_NODE(field->name) << " type=\"D\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"D\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case TYPE_URL:
-								t << "<" << FIX_NODE(field->name) << " type=\"U\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"U\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case TYPE_PASSWORD:
-								t << "<" << FIX_NODE(field->name) << " type=\"P\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"P\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case TYPE_INTEGER:
-								t << "<" << FIX_NODE(field->name) << " type=\"N\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"N\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case TYPE_PHONE_NUMBER:
-								t << "<" << FIX_NODE(field->name) << " type=\"H\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\" type=\"H\">" << FIX_VALUE(field->value) << "</field>\n";
 							  break;
 						  case 'S':
 						  default:
-								t << "<" << FIX_NODE(field->name) << ">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+								t << "<field id=\"" << field->id << "\" name=\"" << FIX_VALUE(field->name) << "\">" << FIX_VALUE(field->value) << "</field>\n";
 	    			  }
 				}
 
+				t << "<creationTime>" << target->creationdate << "</creationTime>\n";
+				t << "<modificationTime>" << target->modificationdate << "</modificationTime>\n";
 				t << "</target>\n";
 
 				target->freeFieldList();
@@ -1725,7 +1776,11 @@ void eWallet::exportXml() {
 			}
 			t << "</targets>\n";
 
-			list.clear();
+//                        Target * target;
+//                        foreach(target, list) {
+//                                delete target;
+//                        }
+                        list.clear();
 
 		    // Close the file
 		    file.close();
