@@ -113,7 +113,8 @@ private slots:
 	void import();
 	void exportData();
 	void exportXml();
-	void textChanged (const QString &);
+        void mergeDB();
+        void textChanged (const QString &);
 	void contextMenuEvent (QContextMenuEvent * event);
 	void renameGroup ();
 	void deleteGroup ();
@@ -146,6 +147,8 @@ private:
 	int isShowGroups;
 	QString currentFilter;
 	QString * currentGroup;
+
+	QString tempPass;
 };
 
 
